@@ -61,13 +61,10 @@ my $date       = $Moment->minus_weeks($FirstDayOfWeek > $Moment->day_of_week)
                         ->with_day_of_week($FirstDayOfWeek);
 
 while ($date->is_before($next_month)) {
-    my @week;
-    foreach my $day (1..7) {
-        if ($date->is_before($this_month)) {
-            push @week, '  ';
-        }
-        elsif ($date->is_before($next_month)) {
-            push @week, sprintf '%2d', $date->day_of_month;
+    my @week = ('  ') x 7;
+    foreach my $index (0..6) {
+        if (!$date->is_before($this_month) && $date->is_before($next_month)) {
+            $week[$index] = sprintf '%2d', $date->day_of_month;
         }
         $date = $date->plus_days(1);
     }
