@@ -5,16 +5,17 @@ use warnings;
 use Carp qw[];
 
 BEGIN {
-    our $VERSION    = '0.43';
+    our $VERSION    = '0.46';
     our @EXPORT_OK  = qw[ NextDayOfWeek
                           NextOrSameDayOfWeek
                           PreviousDayOfWeek
                           PreviousOrSameDayOfWeek
+                          NearestDayOfWeek
                           FirstDayOfWeekInMonth
                           LastDayOfWeekInMonth
                           NthDayOfWeekInMonth
                           WesternEasterSunday
-                          OthodoxEasterSunday
+                          OrthodoxEasterSunday
                           NearestMinuteInterval ];
 
     our %EXPORT_TAGS = (
@@ -60,7 +61,7 @@ sub PreviousDayOfWeek {
 
     return sub {
         my ($tm) = @_;
-        return $tm->minus_days(($tm->day_of_week - $day + 6) % 7 - 1);
+        return $tm->minus_days(($tm->day_of_week - $day + 6) % 7 + 1);
     };
 }
 
@@ -77,6 +78,19 @@ sub PreviousOrSameDayOfWeek {
     };
 }
 
+sub NearestDayOfWeek {
+    @_ == 1 or Carp::croak(q<Usage: NearestDayOfWeek(day)>);
+    my ($day) = @_;
+
+    ($day >= 1 && $day <= 7)
+      or Carp::croak(q<Parameter 'day' is out of the range [1, 7]>);
+
+    return sub {
+        my ($tm) = @_;
+        return $tm->plus_days((($day - $tm->day_of_week + 3) % 7) - 3);
+    };
+}
+
 sub FirstDayOfWeekInMonth {
     @_ == 1 or Carp::croak(q<Usage: FirstDayOfWeekInMonth(day)>);
     my ($day) = @_;
